@@ -24,7 +24,7 @@ from mo_files.url import URL
 from mo_future import PY3, text, is_text
 from mo_logs import Except, Log
 from mo_logs.exceptions import get_stacktrace
-from mo_math.randoms import Random
+from mo_math import randoms
 
 
 class File(object):
@@ -471,7 +471,7 @@ class TempFile(File):
     def __init__(self, filename=None):
         if isinstance(filename, File):
             return
-        self.temp = NamedTemporaryFile(prefix=Random.filename(), delete=False)
+        self.temp = NamedTemporaryFile(prefix=randoms.filename(), delete=False)
         self.temp.close()
         File.__init__(self, self.temp.name)
 
